MODULE mixfunctions

! Two players, with covariates, Nash Equilibrium
!
! This is the module called by mainDGP1.f90
! The program produces the output file quickDGP1.txt
!
! Arie Beresteanu, Ilya Molchanov and Francesca Molinari (Econometrica, 2011): "Sharp Identification Regions in Models with Convex Moment Predictions,"


!This module uses the Nelder-Mead algorithm to find the minimum of the objective function
!The function vfGL allows for covatiates to be taken into account.

IMPLICIT NONE
SAVE

!! Dimensionality constants
INTEGER, PARAMETER       :: GLn=32
INTEGER, PARAMETER       :: cov_num1=3 !the number of covariates that player 1 has plus intercept
INTEGER, PARAMETER       :: cov_num2=3 !the number of covariates that player 2 has plus intercept
INTEGER, PARAMETER       :: par_dim=2+cov_num1+cov_num2 !the total number of parameters equals two thetas plus the number of beta parameters
INTEGER, PARAMETER       :: cov_val1=4 !number of different values that the vector X1 can take
INTEGER, PARAMETER       :: cov_val2=5 !number of different values that the vector X2 can take
!! Other constants
REAL (kind=8), PARAMETER                    :: pi=3.141592653589793D0
REAL (kind=8), PARAMETER                    :: ONE=1.0D0, ZERO=0.0D0, HALF=0.50D0, QUARTER=0.25D0, TWO=2.0D0
REAL (kind=8), PARAMETER                    :: t=1.00D-06
INTEGER, PARAMETER, DIMENSION(2)            :: first_seed=(/90971,854099/)
!! DE variables and parameters
REAL (kind=8), Dimension(par_dim)           :: DLOW=(/-4.25D0,-4.25D0,0.25D0,0.15D0,-0.45D0,-1.35D0,-1.35D0,-0.25D0/) 
REAL (kind=8), Dimension(par_dim)           :: DHIGH=(/-0.250D0,-0.250D0,0.85D0,0.80D0,0.85D0, 0.25D0, 0.25D0,0.35D0/)
INTEGER, PARAMETER                          :: Np=10*par_dim, N_iter=2500
REAL (kind=8), PARAMETER                    :: F=HALF, Cr=0.85D0
!! Misc global variables
REAL, DIMENSION(cov_val1,cov_val2,4)        :: P                            !<--- See coment below
REAL (kind=8)                               :: POP_loc(Np,par_dim)
!!DGP parameters
REAL (kind=8)                               :: X1(cov_val1,cov_num1),X2(cov_val2,cov_num2)
! pick covariates values
DATA X1/-2.00D0,  1.00D0, 0.00D0, -1.50D0, &
         1.00D0, -1.50D0, 0.75D0, -1.00D0, &
         1.00D0,  1.00D0, 1.00D0,  1.00D0/
DATA X2/ 1.00D0, -1.25D0, 0.00D0, 0.60D0,  0.50D0, &
        -1.75D0,  1.00D0, 0.00D0, 0.50D0, -0.50D0, &
         1.00D0,  1.00D0, 1.00D0, 1.00D0,  1.00D0/

!! GL parameters
REAL (kind=8), DIMENSION(GLn)               :: GLx,GLw !Gauss-Legendre grid points and weights

DATA GLx/.001368073D0,.0071942277D0,.0176188818D0,.0325469453D0, &
     .0518394344D0,.0753161897D0,.1027581033D0,.1339089403D0,.1684778666D0, &
     .2061421214D0,.2465500455D0,.2893243619D0,.3340656989D0,.3803563189D0, &
     .4277640192D0,.4758461672D0,.5241538328D0,.5722359808D0,.6196436811D0, &
     .6659343011D0,.7106756381D0,.7534499545D0,.7938578786D0,.8315221334D0, &
     .8660910597D0,.8972418967D0,.9246838103D0,.9481605656D0,.9674530547D0, &
     .9823811182D0,.9928057723D0,.9986319268D0/
DATA GLw/.003509312D0,.0081372115D0,.0126959771D0,.0171370005D0, &
     .0214179378D0,.0254990642D0,.0293420289D0,.0329111118D0,.0361728923D0, &
     .0390969435D0,.0416559573D0,.0438260415D0,.0455869341D0,.0469221942D0, &
     .0478193546D0,.0482700387D0,.0482700387D0,.0478193546D0,.0469221942D0, &
     .0455869341D0,.0438260415D0,.0416559573D0,.0390969435D0,.0361728923D0, &
     .0329111118D0,.0293420289D0,.0254990642D0,.0214179378D0,.0171370005D0, &
     .0126959771D0,.0081372115D0,.0035093120D0/

CONTAINS


!Comment: The variable P is a global variable. The dimensionality of P is
! (# of values that X1 takes) x (# of values that X1 takes) x 4.
! For example: P(m1,m2,4)=P(Y=(1,1)|X1=m1,X2=m2).
! P is computed in the main program and then can be acceced by all the
! procedures below.


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE is_mixABJ(y,ABJ)
! Checks whether (th1,th2) are in the identification region defined by ABJ
! th1,th2 - the two scalar parameters
! P - vector of empirical frequencies
REAL (kind=8), DIMENSION(par_dim), INTENT(IN)   :: y
REAL (kind=8)                                   :: th1,th2,shift1, shift2
INTEGER                                         :: m1,m2
REAL (kind=8), DIMENSION(4)                     :: u,LB,UB
REAL (kind=8), DIMENSION(5)                     :: p_regions
LOGICAL, INTENT(OUT)                            :: ABJ
REAL (kind=8) :: pa, pb, pc,pd,pe,CDF1,CDF2,CDF3,CDF4,pdf1,pdf2,pdf3,PDF4,EX1,EX2

th1=y(1)
th2=y(2)
ABJ=.TRUE.
!check y for all the possible values of the covariates X1 and X2 
DO m1=1,cov_val1  !<--go over all the possible covariate values in X1
DO m2=1,cov_val2  !<--go over all the possible covariate values in X2
    !check whether the candidate y is a feasible parameter
    shift1=DOT_PRODUCT(X1(m1,:),y(3:2+cov_num1))
    shift2=DOT_PRODUCT(X2(m2,:),y(3+cov_num1:par_dim))
    !initial values for the region's probability mass
    !first compute regions
    CDF1=alnorm(-shift1,.FALSE.)
    CDF2=alnorm(-shift2,.FALSE.)
    CDF3=alnorm(-th1-shift1,.FALSE.)
    CDF4=alnorm(-th2-shift2,.FALSE.)
    pdf1=EXP(-(shift1**TWO)/TWO)/SQRT(TWO*pi)
    pdf2=EXP(-(shift2**TWO)/TWO)/SQRT(TWO*pi)
    pdf3=EXP(-((shift1+th1)**TWO)/TWO)/SQRT(TWO*pi)
    pdf4=EXP(-((shift2+th2)**TWO)/TWO)/SQRT(TWO*pi)
    EX1=(pdf1-pdf3)/th1
    EX2=(pdf2-pdf4)/th2

    PA=CDF1*CDF2
    PB=(ONE-CDF1)*CDF2+(CDF4-CDF2)*(ONE-CDF3)
    PC=CDF1*(ONE-CDF2)+(ONE-CDF4)*(CDF3-CDF1)
    PE=(CDF3-CDF1)*(CDF4-CDF2)
    PD=ONE-PA-PB-PC-PE

    !check whether (th1,th2) are in ABJ's ID region
    LB=(/PA,PB,PC,PD/)
    UB=LB+(/ ((ONE+shift1/th1)*(CDF3-CDF1)+EX1)*((ONE+shift2/th2)*(CDF4-CDF2)+EX2), &
            PE,PE,(shift1/th1*(CDF3-CDF1)+EX1)*(shift2/th2*(CDF4-CDF2)+EX2) /)

    IF (P(m1,m2,1)>UB(1)+t .OR. P(m1,m2,2)>UB(2)+t .OR. P(m1,m2,3)>UB(3)+t .OR. P(m1,m2,4)>UB(4)+t) THEN
        ABJ=.FALSE.
    END IF
    IF (.NOT.ABJ) EXIT
END DO !m1
IF (.NOT.ABJ) EXIT
END DO !m2

END SUBROUTINE is_mixABJ
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE is_mixCT(y,CT)
! Checks whether (th1,th2) are in the identification region defined by ABJ
! th1,th2 - the two scalar parameters
! P - vector of empirical frequencies
REAL (kind=8), DIMENSION(par_dim), INTENT(IN)   :: y
REAL (kind=8)                                   :: th1,th2,shift1, shift2
INTEGER                                         :: m1,m2
REAL (kind=8), DIMENSION(4)                     :: u,LB,UB
REAL (kind=8), DIMENSION(5)                     :: p_regions
LOGICAL, INTENT(OUT)                            :: CT
REAL (kind=8) :: pa, pb, pc,pd,pe,CDF1,CDF2,CDF3,CDF4,pdf1,pdf2,pdf3,PDF4,EX1,EX2

th1=y(1)
th2=y(2)
CT=.TRUE.
!check y for all the possible values of the covariates X1 and X2 
DO m1=1,cov_val1  !<--go over all the possible covariate values in X1
DO m2=1,cov_val2  !<--go over all the possible covariate values in X2
    !check whether the candidate y is a feasible parameter
    shift1=DOT_PRODUCT(X1(m1,:),y(3:2+cov_num1))
    shift2=DOT_PRODUCT(X2(m2,:),y(3+cov_num1:par_dim))
    !initial values for the region's probability mass
    !first compute regions
    CDF1=alnorm(-shift1,.FALSE.)
    CDF2=alnorm(-shift2,.FALSE.)
    CDF3=alnorm(-th1-shift1,.FALSE.)
    CDF4=alnorm(-th2-shift2,.FALSE.)
    pdf1=EXP(-(shift1**TWO)/TWO)/SQRT(TWO*pi)
    pdf2=EXP(-(shift2**TWO)/TWO)/SQRT(TWO*pi)
    pdf3=EXP(-((shift1+th1)**TWO)/TWO)/SQRT(TWO*pi)
    pdf4=EXP(-((shift2+th2)**TWO)/TWO)/SQRT(TWO*pi)
    EX1=(pdf1-pdf3)/th1
    EX2=(pdf2-pdf4)/th2

    PA=CDF1*CDF2
    PB=(ONE-CDF1)*CDF2+(CDF4-CDF2)*(ONE-CDF3)
    PC=CDF1*(ONE-CDF2)+(ONE-CDF4)*(CDF3-CDF1)
    PE=(CDF3-CDF1)*(CDF4-CDF2)
    PD=ONE-PA-PB-PC-PE

    !check whether (th1,th2) are in ABJ's ID region
    LB=(/PA,PB,PC,PD/)
    UB=LB+(/ ((ONE+shift1/th1)*(CDF3-CDF1)+EX1)*((ONE+shift2/th2)*(CDF4-CDF2)+EX2), &
              PE,PE,(shift1/th1*(CDF3-CDF1)+EX1)*(shift2/th2*(CDF4-CDF2)+EX2) /)

    IF (P(m1,m2,1)>UB(1)+t .OR. P(m1,m2,2)>UB(2)+t .OR. P(m1,m2,3)>UB(3)+t .OR. P(m1,m2,4)>UB(4)+t) THEN
        CT=.FALSE.
    ELSE
        IF (P(m1,m2,1)<LB(1)-t .OR. P(m1,m2,2)<LB(2)-t .OR. P(m1,m2,3)<LB(3)-t .OR. P(m1,m2,4)<LB(4)-t) THEN
            CT =.FALSE.
        END IF
    END IF
    IF (.NOT.CT) EXIT
END DO !m1
IF (.NOT.CT) EXIT
END DO !m2

END SUBROUTINE is_mixCT
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE is_mixBMM(y,BMM)
! Checks whether (th1,th2) are in the identification region defined by BMM
! th1,th2 - the two scalar parameters
! P - vector of empirical frequencies
REAL (kind=8), DIMENSION(par_dim), INTENT(IN)   :: y
REAL (kind=8)                                   :: th1,th2,shift1, shift2
INTEGER                                         :: m1,m2
!REAL (kind=8), DIMENSION(4)                    :: u,LB,UB
REAL (kind=8), DIMENSION(5)                     :: p_regions
LOGICAL, INTENT(OUT)                            :: BMM
REAL (kind=8) :: pa, pb, pc,pd,pe,CDF1,CDF2,CDF3,CDF4 


th1=y(1)
th2=y(2)
BMM=.TRUE.
!check y for all the possible values of the covariates X1 and X2 
DO m1=1,cov_val1  !<--go over all the possible covariate values in X1
DO m2=1,cov_val2  !<--go over all the possible covariate values in X2
    !check whether the candidate y is a feasible parameter
    shift1=DOT_PRODUCT(X1(m1,:),y(3:2+cov_num1))
    shift2=DOT_PRODUCT(X2(m2,:),y(3+cov_num1:par_dim))
    !initial values for the region's probability mass
    !first compute regions
    CDF1=alnorm(-shift1,.FALSE.)
    CDF2=alnorm(-shift2,.FALSE.)
    CDF3=alnorm(-th1-shift1,.FALSE.)
    CDF4=alnorm(-th2-shift2,.FALSE.)

    PA=CDF1*CDF2
    PB=(ONE-CDF1)*CDF2+(CDF4-CDF2)*(ONE-CDF3)
    PC=CDF1*(ONE-CDF2)+(ONE-CDF4)*(CDF3-CDF1)
    PE=(CDF3-CDF1)*(CDF4-CDF2)
    PD=ONE-PA-PB-PC-PE

    p_regions=(/PA, PB, PC, PD, PE/)
    BMM=isminneg2(th1,th2,shift1,shift2,p_regions,m1,m2) !check if the point is in BMM's ID region only if don't already know that it can be
    IF (.NOT.BMM) EXIT
END DO !m1
IF (.NOT.BMM) EXIT
END DO !m2

END SUBROUTINE is_mixBMM
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
LOGICAL FUNCTION isminneg2(th1,th2,shift1,shift2,p_reg,m1,m2)
!Neldel-Mead algorithm to check whether the minimum of the value function
!is negative or not

REAL (kind=8), INTENT(IN)               :: th1,th2,shift1,shift2
REAL (kind=8), INTENT(IN), DIMENSION(5) :: p_reg
INTEGER, INTENT(IN)                     :: m1,m2

REAL (kind=8), DIMENSION(5,5) :: U
REAL (kind=8), DIMENSION(4) :: M,R,E,C,S
REAL (kind=8) :: zM,zR,zE,zC,zS
INTEGER :: j,ii,jj

U(1,:)=ZERO
U(2,1:4)=(/ONE, -HALF, ZERO, HALF/)
U(3,1:4)=(/ZERO, ONE, -ONE, ZERO/)
U(4,1:4)=(/ZERO, ZERO, ONE, -ONE/)
U(5,1:4)=(/-ONE, ZERO, ZERO, ONE/)

DO j=2,5
    U(j,5)=vfGL(U(j,1:4),th1,th2,shift1,shift2,p_reg,m1,m2)
END DO


! order the points by the fifth column
CALL inssor(U)
isminneg2=(U(5,5)>=-t)
!PRINT *,isminneg2
DO
    !PRINT *,cc,U(5,5),U(1,5)
    IF (.NOT.(isminneg2) .OR. ((U(5,5)-U(1,5))<t/TWO)) EXIT
    ! defining new points
    M=(U(1,1:4)+U(2,1:4)+U(3,1:4)+U(4,1:4))/4.0D0 !average of all but the worse point
    zM=vfGL(M,th1,th2,shift1,shift2,p_reg,m1,m2)
    R=TWO*M-U(5,1:4)
    zR=vfGL(R,th1,th2,shift1,shift2,p_reg,m1,m2)
    IF (zR<U(4,5))  THEN  !case 1
        IF (U(1,5)<zR) THEN
            U(5,1:4)=R
            U(5,5)= zR
        ELSE
            E=TWO*R-M
            zE=vfGL(E,th1,th2,shift1,shift2,p_reg,m1,m2)
            IF (zE<U(1,5)) THEN
                U(5,1:4)=E
                U(5,5)=zE
            ELSE
                U(5,1:4)=R
                U(5,5)=zR
            END IF
        END IF
    ELSE   !case 2
        IF (zR<U(5,5)) THEN
            U(5,1:4)=R
            U(5,5)=zR
        END IF
        C=(U(5,1:4)+M)/TWO
        zC=vfGL(C,th1,th2,shift1,shift2,p_reg,m1,m2)
        IF (zC<U(5,5)) THEN
            U(5,1:4)=C
            U(5,5)=zC
        ELSE
            S=(U(1,1:4)+U(5,1:4))/TWO
            zS=vfGL(S,th1,th2,shift1,shift2,p_reg,m1,m2)
            U(5,1:4)=S
            U(5,5)=zS
            U(4,1:4)=M
            U(4,5)=zM
        END IF
    END IF
    ! order the points to create BGW
    CALL inssor(U)
    isminneg2=(U(5,5)>=-t)
END DO

END FUNCTION isminneg2
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
REAL (kind=8) FUNCTION vfGL(u,th1,th2,shift1,shift2,pp,m1,m2)

REAL (kind=8), INTENT(IN)  :: u(4),th1,th2,shift1,shift2,pp(5)
INTEGER , INTENT(IN)       :: m1,m2
REAL (kind=8)              :: u1,u2,u3,u4,a,b,c,um,E_singleton,E_GL
REAL (kind=8)              :: k,jj,g,AA,BB,CC,DD,integrand_tot,int_value,temp1,temp2
INTEGER                    :: i

!Created: 04/07/10
!Last modified: 04/15/10
!
! This function evaluates E(h(Q(S_{\theta})),u) using quadrature and
! solving one of the two integrals in closed form.  See
! speed_improvements_algorithm.tex for details

u1 = u(1)
u2 = u(2)
u3 = u(3)
u4 = u(4)
a  = u1-u2-u3+u4
b  = u3-u1
c  = u2-u1
um = MAX(u2,u3)

E_singleton = u1*pp(1)+u2*pp(2)+u3*pp(3)+u4*pp(4)

!quadrature part:
E_GL = ZERO
DO i=1,32
    k = (a*GLx(i) + b)
    jj= (c*GLx(i) + u1)
    g = (um-u1-c*GLx(i))/k

    CC = alnorm(-shift1-th1*MIN(g,ONE),.FALSE.)-alnorm(-shift1,.FALSE.)
    DD = alnorm(-shift1-th1,.FALSE.)-alnorm(-shift1+MAX(ZERO,-th1*g),.FALSE.)
    AA=(jj+k*shift1/(-th1))*DD+(k/(-th1))*(normpdf(-shift1+MAX(ZERO,-th1*g))-normpdf(-shift1-th1))
    BB=(jj+k*shift1/(-th1))*CC+(k/(-th1))*(normpdf(-shift1)-normpdf(-shift1-th1*MIN(g,ONE)))
    integrand_tot=ZERO
    IF ((k .GT. ZERO).AND.(g .GT. ZERO)) integrand_tot=integrand_tot+um*cc
    IF ((k .LT. ZERO).AND.(g .LT. ONE)) integrand_tot=integrand_tot+um*DD
    IF ((k .GT. ZERO).AND.(g .LT. ONE)) integrand_tot=integrand_tot+AA
    IF ((k .LT. ZERO).AND.(g .GT. ZERO)) integrand_tot=integrand_tot+BB
    !integrand_tot = um*(k .GT. ZERO)*(g .GT. ZERO)*CC + um*(k .LT. ZERO)*(g .LT. ONE)*DD + &
    !    (k .GT. ZERO)*(g .LT. ONE)*AA + (k .LT. ZERO)*(g .GT. ZERO)*BB
    E_GL = E_GL+integrand_tot*normpdf(GLx(i)*(-th2)-shift2)*GLw(i)
END DO
E_GL=E_GL*(-th2)

vfGL=E_singleton + E_GL - DOT_PRODUCT(P(m1,m2,:),u)

END FUNCTION vfGL
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE inssor(U)
!  Sorts U into increasing order (Insertion sort) by the fifth column
! this procedure is used by Nelder-Mead's algorithm

      REAL (kind=8), DIMENSION (5,5), Intent (InOut) :: U
      REAL (kind=8), DIMENSION (5) :: XWRK, XMIN
      INTEGER :: ICRS, IDCR
!
! We first bring the minimum to the first location in the array.
! That way, we will have a "guard", and when looking for the
! right place to insert a value, no loop test is necessary.
!
      IF (U(1,5) < U(5,5)) THEN
          XMIN = U(1,1:5)
      ELSE
          XMIN = U(5,1:5)
          U(5,1:5) = U(1,1:5)
      END IF
      DO IDCR = 4, 2, -1
         XWRK = U(IDCR,1:5)
         IF (XWRK(5) < XMIN(5)) THEN
            U(IDCR,1:5) = XMIN
            XMIN = XWRK
         END IF
      END DO
      U(1,1:5) = XMIN
!
! The first value is now the minimum
! Loop over the array, and when a value is smaller than
! the previous one, loop down to insert it at its right place.
!
      DO ICRS = 3, 5
         XWRK = U(ICRS,1:5)
         IDCR = ICRS - 1
         IF (XWRK(5) < U(IDCR,5)) THEN
            U(ICRS,1:5) = U(IDCR,1:5)
            IDCR = IDCR - 1
            DO
               IF (XWRK(5) >= U(IDCR,5)) EXIT
               U(IDCR+1,1:5) = U(IDCR,1:5)
               IDCR = IDCR - 1
            END DO
            U(IDCR+1,1:5) = XWRK
         END IF
      END DO
!
      RETURN
!
END SUBROUTINE inssor
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
FUNCTION alnorm ( x, upper )
!**************************************************************************
!
!! ALNORM computes the cumulative density of the standard normal distribution.
!
!  Modified:
!
!    13 January 2008
!
!  Author:
!
!    David Hill
!    FORTRAN90 version by John Burkardt
!
!  Reference:
!
!    David Hill,
!    Algorithm AS 66:
!    The Normal Integral,
!    Applied Statistics,
!    Volume 22, Number 3, 1973, pages 424-427.
!
!  Parameters:
!
!    Input, real ( kind = 8 ) X, is one endpoint of the semi-infinite interval
!    over which the integration takes place.
!
!    Input, logical UPPER, determines whether the upper or lower
!    interval is to be integrated:
!    .TRUE.  => integrate from X to + Infinity;
!    .FALSE. => integrate from - Infinity to X.
!
!    Output, real ( kind = 8 ) ALNORM, the integral of the standard normal
!    distribution over the desired interval.
!
  IMPLICIT NONE

  REAL (kind=8), PARAMETER :: a1 = 5.75885480458D+00
  REAL (kind=8), PARAMETER :: a2 = 2.62433121679D+00
  REAL (kind=8), PARAMETER :: a3 = 5.92885724438D+00
  REAL (kind=8) alnorm
  REAL (kind=8), PARAMETER :: b1 = -29.8213557807D+00
  REAL (kind=8), PARAMETER :: b2 = 48.6959930692D+00
  REAL (kind=8), PARAMETER :: c1 = -0.000000038052D+00
  REAL (kind=8), PARAMETER :: c2 = 0.000398064794D+00
  REAL (kind=8), PARAMETER :: c3 = -0.151679116635D+00
  REAL (kind=8), PARAMETER :: c4 = 4.8385912808D+00
  REAL (kind=8), PARAMETER :: c5 = 0.742380924027D+00
  REAL (kind=8), PARAMETER :: c6 = 3.99019417011D+00
  REAL (kind=8), PARAMETER :: con = 1.28D+00
  REAL (kind=8), PARAMETER :: d1 = 1.00000615302D+00
  REAL (kind=8), PARAMETER :: d2 = 1.98615381364D+00
  REAL (kind=8), PARAMETER :: d3 = 5.29330324926D+00
  REAL (kind=8), PARAMETER :: d4 = -15.1508972451D+00
  REAL (kind=8), PARAMETER :: d5 = 30.789933034D+00
  REAL (kind=8), PARAMETER :: ltone = 7.0D+00
  REAL (kind=8), PARAMETER :: p = 0.398942280444D+00
  REAL (kind=8), PARAMETER :: q = 0.39990348504D+00
  REAL (kind=8), PARAMETER :: r = 0.398942280385D+00
  LOGICAL :: up
  LOGICAL :: upper
  REAL (kind=8), PARAMETER :: utzero = 18.66D+00
  REAL (kind=8) x
  REAL (kind=8) y
  REAL (kind=8) z

  up = upper
  z = x

  IF ( z < 0.0D+00 ) THEN
    up = .not. up
    z = - z
  END IF

  IF ( ltone < z .and. ( ( .not. up ) .or. utzero < z ) ) THEN

    IF ( up ) THEN
      alnorm = 0.0D+00
    ELSE
      alnorm = 1.0D+00
    END IF

    RETURN

  END IF

  y = 0.5D+00 * z * z

  IF ( z <= con ) THEN

    alnorm = 0.5D+00 - z * ( p - q * y &
      / ( y + a1 + b1 &
      / ( y + a2 + b2 & 
      / ( y + a3 ))))

  ELSE

    alnorm = r * exp ( - y ) &
      / ( z + c1 + d1 &
      / ( z + c2 + d2 &
      / ( z + c3 + d3 &
      / ( z + c4 + d4 &
      / ( z + c5 + d5 &
      / ( z + c6 ))))))

  END IF

  IF ( .not. up ) THEN
    alnorm = 1.0D+00 - alnorm
  END IF

  RETURN
END FUNCTION alnorm
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
FUNCTION norminv ( p )

!**************************************************************************
!
!! norminv inverts the standard normal CDF.
!
!  Discussion:
!
!    The result is accurate to about 1 part in 10**16.
!
!  Modified:
!
!    27 December 2004
!
!  Author:
!
!    Michael Wichura.
!    FORTRAN90 version by John Burkardt
!
!  Reference:
!
!    Michael Wichura,
!    The Percentage Points of the Normal Distribution,
!    Algorithm AS 241,
!    Applied Statistics,
!    Volume 37, Number 3, pages 477-484, 1988.
!
!  Parameters:
!
!    Input, real ( kind = 8 ) P, the value of the cumulative probability 
!    densitity function.  0 < P < 1.  If P is outside this range,
!    an "infinite" value will be returned.
!
!    Output, real ( kind = 8 ) D_NORMAL_01_CDF_INVERSE, the normal deviate 
!    value with the property that the probability of a standard normal 
!    deviate being less than or equal to the value is P.
!
  IMPLICIT NONE

  REAL    ( kind = 8 ), PARAMETER, DIMENSION ( 8 ) :: a = (/ &
    3.3871328727963666080D+00, &
    1.3314166789178437745D+02, &
    1.9715909503065514427D+03, &
    1.3731693765509461125D+04, &
    4.5921953931549871457D+04, &
    6.7265770927008700853D+04, &
    3.3430575583588128105D+04, &
    2.5090809287301226727D+03 /)
  REAL    ( kind = 8 ), PARAMETER, DIMENSION ( 8 ) :: b = (/ &
    1.0D+00, &
    4.2313330701600911252D+01, &
    6.8718700749205790830D+02, &
    5.3941960214247511077D+03, &
    2.1213794301586595867D+04, &
    3.9307895800092710610D+04, &
    2.8729085735721942674D+04, &
    5.2264952788528545610D+03 /)
  REAL   ( kind = 8 ), PARAMETER, DIMENSION ( 8 ) :: c = (/ &
    1.42343711074968357734D+00, &
    4.63033784615654529590D+00, &
    5.76949722146069140550D+00, &
    3.64784832476320460504D+00, &
    1.27045825245236838258D+00, &
    2.41780725177450611770D-01, &
    2.27238449892691845833D-02, &
    7.74545014278341407640D-04 /)
  REAL    ( kind = 8 ), PARAMETER :: const1 = 0.180625D+00
  REAL    ( kind = 8 ), PARAMETER :: const2 = 1.6D+00
  REAL    ( kind = 8 ), PARAMETER, DIMENSION ( 8 ) :: d = (/ &
    1.0D+00, &
    2.05319162663775882187D+00, &
    1.67638483018380384940D+00, &
    6.89767334985100004550D-01, &
    1.48103976427480074590D-01, &
    1.51986665636164571966D-02, &
    5.47593808499534494600D-04, &
    1.05075007164441684324D-09 /)
  REAL    ( kind = 8 ), PARAMETER, DIMENSION ( 8 ) :: e = (/ &
    6.65790464350110377720D+00, &
    5.46378491116411436990D+00, &
    1.78482653991729133580D+00, &
    2.96560571828504891230D-01, &
    2.65321895265761230930D-02, &
    1.24266094738807843860D-03, &
    2.71155556874348757815D-05, &
    2.01033439929228813265D-07 /)
  REAL    ( kind = 8 ), PARAMETER, DIMENSION ( 8 ) :: f = (/ &
    1.0D+00, &
    5.99832206555887937690D-01, &
    1.36929880922735805310D-01, &
    1.48753612908506148525D-02, &
    7.86869131145613259100D-04, &
    1.84631831751005468180D-05, &
    1.42151175831644588870D-07, &
    2.04426310338993978564D-15 /)
  REAL    ( kind = 8 ) p
  REAL    ( kind = 8 ) q
  REAL    ( kind = 8 ) r
  REAL    ( kind = 8 ) norminv 
  !REAL    ( kind = 8 ) r8poly_value
  REAL    ( kind = 8 ), PARAMETER :: split1 = 0.425D+00
  REAL    ( kind = 8 ), PARAMETER :: split2 = 5.0D+00

  IF ( p <= 0.0D+00 ) THEN
    norminv = - HUGE ( p )
    RETURN
  END IF

  IF ( 1.0D+00 <= p ) THEN
    norminv = HUGE ( p )
    RETURN
  END IF

  q = p - 0.5D+00

  IF ( ABS ( q ) <= split1 ) THEN

    r = const1 - q * q
    norminv = q * r8poly_value ( 8, a, r ) &
                                 / r8poly_value ( 8, b, r )

  ELSE

    IF ( q < 0.0D+00 ) THEN
      r = p
    ELSE
      r = 1.0D+00 - p
    END IF

    IF ( r <= 0.0D+00 ) THEN
      norminv = - 1.0D+00
      STOP
    END IF

    r = sqrt ( -log ( r ) )

    IF ( r <= split2 ) THEN

      r = r - const2
      norminv = r8poly_value ( 8, c, r ) &
                               / r8poly_value ( 8, d, r )

    ELSE

      r = r - split2
      norminv = r8poly_value ( 8, e, r ) &
                               / r8poly_value ( 8, f, r )
   
    END IF

    IF ( q < 0.0D+00 ) THEN
      norminv = - norminv
    END IF

  END IF

  RETURN
END FUNCTION norminv

FUNCTION r8poly_value ( n, a, x )

!**************************************************************************
!
!! R8POLY_VALUE evaluates an R8POLY
!
!  Discussion:
!
!    For sanity's sake, the value of N indicates the NUMBER of 
!    coefficients, or more precisely, the ORDER of the polynomial,
!    rather than the DEGREE of the polynomial.  The two quantities
!    dIFfer by 1, but cause a great deal of confusion.
!
!    Given N and A, the form of the polynomial is:
!
!      p(x) = a(1) + a(2) * x + ... + a(n-1) * x^(n-2) + a(n) * x^(n-1)
!
!  ModIFied:
!
!    13 August 2004
!
!  Author:
!
!    John Burkardt
!
!  Parameters:
!
!    Input, INTEGER ( kind = 4 ) N, the order of the polynomial.
!
!    Input, REAL ( kind = 8 ) A(N), the coefficients of the polynomial.
!    A(1) is the constant term.
!
!    Input, REAL ( kind = 8 ) X, the point at which the polynomial is 
!    to be evaluated.
!
!    Output, REAL ( kind = 8 ) R8POLY_VALUE, the value of the polynomial at X.
!
  IMPLICIT NONE

  INTEGER ( kind = 4 ) n

  REAL    ( kind = 8 ) a(n)
  INTEGER ( kind = 4 ) i
  REAL    ( kind = 8 ) r8poly_value
  REAL    ( kind = 8 ) x

  r8poly_value = 0.0D+00
  DO i = n, 1, -1
    r8poly_value = r8poly_value * x + a(i)
  END DO

  RETURN
END FUNCTION r8poly_value
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
REAL (kind=8) FUNCTION normpdf(x)

REAL (kind=8), INTENT(IN) :: x

normpdf=(ONE/SQRT(TWO*pi))*EXP(-x**TWO/TWO)

END FUNCTION normpdf
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE pick_three(indx_exclude,indx)
! this subroutine randomly picks three indexes from (/1,2,...,Np/) but the
! number indx_exclude cannot be picked
INTEGER, INTENT(IN)  :: indx_exclude
INTEGER, INTENT(OUT) :: indx(3)
REAL(kind=8)         :: r
INTEGER              :: temp1(Np-1),temp2(Np-2),temp3(Np-3),t,i

!first get rid of the index indx_exclude
temp1=(/(i,i=1,indx_exclude-1),(i,i=indx_exclude+1,Np)/)

!pick first index
CALL RANDOM_NUMBER(r)
t=FLOOR((Np-1)*r)+1
indx(1)=temp1(t)

temp2=(/temp1(1:t-1),temp1(t+1:Np-1)/)
!pick 2nd index
CALL RANDOM_NUMBER(r)
t=FLOOR((Np-2)*r)+1
indx(2)=temp2(t)

temp3=(/temp2(1:t-1),temp2(t+1:Np-2)/)
!pic 3rd index
CALL RANDOM_NUMBER(r)
t=FLOOR((Np-3)*r)+1
indx(3)=temp3(t)

END SUBROUTINE pick_three
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE selection(th1,th2,shift1,shift2,alpha_01,alpha_10,Pout)
! this procedure creates a selection
REAL(kind=8), INTENT(in)  :: th1,th2,shift1,shift2,alpha_01,alpha_10
REAL(kind=8), intent(out) :: Pout(4)
REAL(kind=8)              :: CDF1,CDF2,CDF3,CDF4,pdf1,pdf2,pdf3,pdf4,EX1,EX2
REAL(kind=8)              :: E1in,E1out,E2in,E2out
REAL(kind=8)              :: PA,PB,PC,PD,PE,P00,P01,P10,P11,alpha_m,p_reg(5)

!Compute the probabilities of the various outcomes
!
!                                |
!            PC                  |      PD
!        -x2*beta2-theta2+-------+-----------------
!                        |       |
!                        |  PE   |
!    -------------x2*beta2-------+
!                    x1*beta1  -theta1-x1*beta1
!           PA           |         
!                        |           PB
!                        |
!

CDF1=alnorm(-shift1,.FALSE.)
CDF2=alnorm(-shift2,.FALSE.)
CDF3=alnorm(-th1-shift1,.FALSE.)
CDF4=alnorm(-th2-shift2,.FALSE.)
pdf1=EXP(-(shift1**TWO)/TWO)/SQRT(TWO*pi)
pdf2=EXP(-(shift2**TWO)/TWO)/SQRT(TWO*pi)
pdf3=EXP(-((shift1+th1)**TWO)/TWO)/SQRT(TWO*pi)
pdf4=EXP(-((shift2+th2)**TWO)/TWO)/SQRT(TWO*pi)
EX1=(pdf1-pdf3)/th1
EX2=(pdf2-pdf4)/th2
alpha_m=ONE-alpha_01-alpha_10

PA=CDF1*CDF2
PB=(1-CDF1)*CDF2+(CDF4-CDF2)*(ONE-CDF3)
PC=CDF1*(1-CDF2)+(ONE-CDF4)*(CDF3-CDF1)
PE=(CDF3-CDF1)*(CDF4-CDF2)
PD=ONE-PA-PB-PC-PE
!PRINT *,PA,PB,PC,PD,PE
p_reg=(/PA,PB,PC,PD,PE/)

E1in  = (shift2*(CDF4-CDF2)+normpdf(-shift2)-normpdf(-shift2-th2))/(-th2);
E1out = (ONE+shift2/th2)*(CDF4-CDF2)+(normpdf(-shift2)-normpdf(-shift2-th2))/th2;
E2in  = (shift1*(CDF3-CDF1)+normpdf(-shift1)-normpdf(-shift1-th1))/(-th1);
E2out = (ONE+shift1/th1)*(CDF3-CDF1)+(normpdf(-shift1)-normpdf(-shift1-th1))/th1;

!PRINT *,PE

P00=PA+alpha_m*E1out*E2out;
P10=PB+alpha_10*PE+alpha_m*E1in*E2out;
P01=PC+alpha_01*PE+alpha_m*E1out*E2in;
!P11=PD+alpha_m*E1in*E2in;
P11=ONE-P00-P10-P01

Pout=(/P00,P10,P01,P11/)

END SUBROUTINE selection
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
SUBROUTINE make_y(i,y,METHOD)
! creates an alternative to current using a combination of a,b,c and puts
! it into y.

INTEGER, INTENT(IN)                            :: i
INTEGER, INTENT(IN)                            :: METHOD
REAL (kind=8), DIMENSION(par_dim), INTENT(OUT) :: y
REAL (kind=8), DIMENSION(par_dim)              :: a,b,c,current
REAL (kind=8)                                  :: rj,h,delta
INTEGER                                        :: k, j, indx(3)

current=POP_loc(i,:)
CALL RANDOM_NUMBER(delta)
delta=(delta-HALF)/5.0D0

!pick 3 other agents a,b and c
CALL pick_three(i,indx)
a=POP_loc(indx(1),1:par_dim)
b=POP_loc(indx(2),1:par_dim)
c=POP_loc(indx(3),1:par_dim)

SELECT CASE (METHOD)
CASE (1)
!!!!!!!!!!!!
!Box method!
!!!!!!!!!!!!
    y=MAX(MIN(a+(F+delta)*(b-c),DHIGH),DLOW)
    !!one random coordinate, k, to be replaced for sure
    CALL RANDOM_NUMBER(h)
    k=FLOOR(REAL(par_dim,kind=8)*h)+1 !pick a random coordinate 
    DO j=1,par_dim
        IF (j.NE.k) THEN
            !!the other coordinates are replaced if rj<Cr
            CALL RANDOM_NUMBER(rj)
            IF (rj.GT.Cr) THEN
                y(j)=current(j)
            END IF
        END IF
    END DO !j

CASE (2)
!!!!!!!!!!!!!!
!Torus method!
!!!!!!!!!!!!!!
    y=a+(F+delta)*(b-c)
    !!one random coordinate, k, to be replaced for sure
    CALL RANDOM_NUMBER(h)
    k=FLOOR(REAL(par_dim,kind=8)*h)+1 !pick a random coordinate 
    DO j=1,par_dim
        IF (y(j)<DLOW(j))  y(j)=(DHIGH(j)-DLOW(j))+y(j)
        IF (y(j)>DHIGH(j)) y(j)=y(j)-(DHIGH(j)-DLOW(j))
        IF (j.NE.k) THEN
        !!the other coordinates are replaced if rj<Cr
        CALL RANDOM_NUMBER(rj)
            IF (rj.GT.Cr) THEN
                y(j)=current(j)
            END IF
        END IF
    END DO !j
END SELECT

END SUBROUTINE make_y
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


END MODULE mixfunctions